######################################################################
#
# EPrints::Search::Condition::Control
#
######################################################################
#
#
# Copyright 2000-2008 University of Southampton. All Rights Reserved.
# 
#  This file is part of GNU EPrints 3.
#  
#  Copyright (c) 2000-2008 University of Southampton, UK. SO17 1BJ.
#  
#  EPrints 3 is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  EPrints 3 is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with EPrints 3; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
######################################################################

=pod

=head1 NAME

B<EPrints::Search::Condition::Control> - Control structure

=head1 DESCRIPTION

Intersect the results of sub-conditions.

=cut

package EPrints::Search::Condition::Control;

use EPrints::Search::Condition;

@ISA = qw( EPrints::Search::Condition );

use strict;

sub new
{
	my( $class, @params ) = @_;

	EPrints::abort( "new called on abstract Control condition.");
}

# internal means don't strip canpass off the front.
# nb. this is only good for AND and OR. Not would need a custom version of this.
sub optimise
{
	my( $self, %opts ) = @_;

	my $tree = $self;

	my @new_sub_ops = ();
	foreach my $sub_op ( @{$tree->{sub_ops}} )
	{
		push @new_sub_ops, $sub_op->optimise( %opts );
	}
	$tree->{sub_ops} = \@new_sub_ops;

	# flatten sub opts with the same type
	# so OR( A, OR( B, C ) ) becomes OR(A,B,C)
	my $flat_ops = [];
	foreach my $sub_op ( @{$tree->{sub_ops}} )
	{
		if( $sub_op->{op} eq $tree->{op} )
		{
			push @{$flat_ops}, @{$sub_op->{sub_ops}};
		}
		else
		{
			push @{$flat_ops}, $sub_op;
		}
	}
	$tree->{sub_ops} = $flat_ops;

	# control-specific condition stuff
	$tree = $tree->optimise_specific( %opts );


	# no items, match nothing.
	if( !defined $tree->{sub_ops} || scalar @{$tree->{sub_ops}} == 0 )
	{
		return EPrints::Search::Condition::True->new();
	}

	# only one sub option, just return it.
	if( scalar @{$tree->{sub_ops}} == 1 )
	{
		return $tree->{sub_ops}->[0];
	}

	return $tree;
}

1;
